--//------------------------A--算法搜索地图中的最短路径-----------------------------

local aimPx	= 0 					--目标点
local aimPy = 0
local startPx = 0						--起始点
local startPy = 0
local openArr = {}					--开启队列
local closeArr = {}					--关闭队列
local mapArr = {}					--地图数据
local curPx	= 0					--当前点,即当前搜索到的点
local curPy = 0
local pDic = {}					--点数据集,
local DET = 10				--边相邻格子之间的默认距离
local hasRoad = false					--是否有找到通路

--
 -- 返回点[px,py]的数据在pDic中的标示
 -- @param px
 -- @param py
 -- @return 
 -- 
 --/				
function pKey(px,py)
	return px .."_"..py
end

--查找arr中是否存在value
local function checkExist(arr , value)
	for i,v in pairs(arr) do
		if v == value then
			--print(tabletostring(arr),i)
			return i
		end
	end
	--print(tabletostring(arr),-1)
	return -1
end

--
 -- 把key加入关闭队列，并从开启队列中删除 
 -- @param key
 -- 
 --/		
local function pushToClose(key)
	if checkExist(closeArr,key) == -1 then
		table.insert(closeArr,key)
	end
	local index = checkExist(openArr,key)
	if (index >= 0) then
		table.remove(openArr,index)
	end
end

--
 -- 返回点[px,py]到当前点[curPx,curPy]点间的距离
 --/
local function getDet(px,py)
	local curKey = pKey(curPx,curPy)
	local arr = mapArr[curKey]--当前点周围的点集
	for _,item in pairs(arr) do
		if (item[1] == px and item[2] == py) then
			return item[3]
		end
		
	end
	return DET
end


--
 -- 启式函数
 -- 点[px,py]到终点的猜测距离 
 -- @param px 
 -- @param py
 -- @return 
 -- 
 --/		
local function h(px,py)
	local det = 0
	local ty0
	local ty1
--	if (aimPx%2 ~= 0) then
--		ty1 = aimPy + 0.5
--	else
		ty1 = aimPy
--	end
--	if (px%2 ~= 0) then
--		ty0 = py + 0.5
--	else
		ty0 = py
--	end
	det = (math.abs(px - aimPx) / 2)*DET + math.abs(ty0 - ty1)*DET
	return det
end

--
 -- 返回一个新加入开启队列的点 
 -- @param px 新点的坐标,==-1,
 -- @param py
 -- @return 
 -- 
 --/		
local function newNode(px,py)
	local p = {}
	local curKey = pKey(curPx,curPy)--当前点的标示
	if pDic[curKey] ~= nil then
		p.x = px
		p.y = py
		p.g = pDic[curKey].g + getDet(px,py)--起始点到此新点的距离等于当前点[curPx,curPy]到超始的距离加1，
		p.p = curKey--此新点的父级为当前点[curPx,curPy]
		p.h = h(px,py)--此新点离终点的估值
		p.f = p.g + p.h--经过此点到终点的大致值
	else
		--此时是起始点
		p.x = startPx
		p.y = startPy
		p.g = 0
		p.h = 0
		p.p = nil
		p.f = 0
	end
	return p
end

--
 -- 搜索到点[px,py] 
 -- @param px 
 -- @param py
 -- 
 --/		
local function seekP(px,py)
	--print("seekp:3:",px,py)
	if (px == aimPx and py == aimPy) then
		--已经搜索到最短路径
		hasRoad = true
	else
		curPx = px
		curPy = py
		
		local key = 0
		local curKey = pKey(curPx,curPy)
		local arr = mapArr[curKey]--当前点周围的点集
		local curP = pDic[curKey]
		if arr == nil then
			print("seekp::",curPx,curPy)
			for k,v in pairs(mapArr) do
				print(k,tabletostring(v))
			end
		end
		for _,item in pairs(arr) do--item=[x,y,当前点到这个点的距离(对角线是14,其它是10)]
			key = pKey(item[1],item[2])
			--print("seekp:1:",item[1],item[2])
			if checkExist(closeArr,key) == -1 then
				--此点不在关闭队列中
				
				if checkExist(openArr,key) == -1 then
					--此点不在开启队列中
					--则加入开启队列
					table.insert(openArr,key)
					pDic[key] = newNode(item[1],item[2])
				else
					--此点在开启队列中
					--则需要判断当前搜索到此点的距离是否比之前的值小，即当前的g值是否小于之前的g值
					--如果比之前的g值小，则把此点的g点改成当前的，并把此父级改成当前点
					--反之不处理
					local p = pDic[key]
					if (p.g > (curP.g + item[3])) then
						p.g = (curP.g + item[3])
						p.p = curKey
						pDic[key] = p
					end
				end
			end
		end
		
		pushToClose(curKey)
		
		if (#openArr > 0) then
			local minObj = nil
			--获取开启队列中f值最小的值
			for _,item in pairs(openArr) do
				local po = pDic[item]
				if (minObj == nil or po.f < minObj.f) then
					minObj = po
				end
			end
			seekP(minObj.x,minObj.y)
		else
			--开启队列为空,则没有找到路径
			hasRoad = false
		end
	end
end

local function getRoad()
	local road = {}
	local key = pKey(aimPx,aimPy)
	local p = pDic[key]
	while p and p.p do
		table.insert(road,1,{p.x,p.y})
		key = p.p
		p = pDic[key]
	end
	return road
end

--
 -- 开始搜索 
 -- 需初始化一些数据
 --/		
local function startSeek()
	openArr = {}
	closeArr = {}
	pDic = {}
	curPx = -1					--刚开始搜索时
	curPy = -1
	
	local key = pKey(startPx,startPy)
	table.insert(openArr,key)
	pDic[key] = newNode(startPx,startPy)
	
	seekP(startPx,startPy)
end
























--
 -- 在_mapArr上搜索从点[x0,y0]到点[x1,y1]的路径
 -- @param _mapArr 地图数据
 -- @param x0 起始点
 -- @param y0
 -- @param x1 目标点
 -- @param y1
 -- 
 --/			
function seek(_mapArr,x0,y0,x1,y1)
	mapArr = _mapArr
	startPx = x0
	startPy = y0
	aimPx = x1
	aimPy = y1
	
	startSeek()
	if (hasRoad == true) then
		return getRoad()
	end
	--print("没有通路")
	return nil
end

--测试
--local cardarr = {}
--local function flatPath(px , py)
--	local arr = {}
--	local tx
--	local ty
--	for i=-1,1,1 do
--		for j=-1,1,1 do
--			tx = px + i
--			ty = py + j
--			if i ~= j and tx >=0 and tx <= 5 and ty >= 0 and ty <= 9 and cardarr[pKey(tx,ty)] == nil then--fight_getmaxrow() then--i==j是对角
--				print("flatPath:",tx,ty)
--				table.insert(arr,{tx,ty,10})
--			end
--		end
--	end
--	return arr
--end
--
--local arrtest = {}
--for i=0,5 do
--	for j=0,9 do
--		arrtest[pKey(i,j)] = flatPath(i,j)
--	end
--end
--print("----------------------------seekpath--------------------")
--for k,v in pairs(arrtest) do
--	print(k,tabletostring(v))
--end
--
--local r = seek(arrtest,0,0,5,5)
--
--for i,v in pairs(r) do
--	print(i.."::",tabletostring(v))
--end